#include "GeoTreeSimplifier.h"
#include "Hoja.h"

#include <iostream>
#include <fstream>
#include <ctime>

using namespace Geometry;
using namespace std;

TreeSimplifier::TreeSimplifier(	const	Mesh	*m,
																TIPOFUNC		upb)
{
	objmesh						=	m;
	mtreesimpsequence	=	new Geometry::TreeSimplificationSequence();
	//mtreesimpsequence=NULL;
	activas	=	0;
	counth	=	0;

	// Mesh de salida
	mesh	=	new Geometry::Mesh();
	*mesh	=	*m;

	//	Sets the progress bar.
	mUPB	=	upb;
}

TreeSimplifier::~TreeSimplifier()
{
	delete	mtreesimpsequence;
}

// Returns the simplified mesh.
Geometry::Mesh *TreeSimplifier::GetMesh ()
{
	return	mesh;
}

Geometry::TreeSimplificationSequence *TreeSimplifier::GetSimplificationSequence()
{
	return	mtreesimpsequence;
}

void TreeSimplifier::Simplify(Real lodfactor,	Index meshLeaves)
{
	// paramlod indica el nmero de vrtices
	long	int	totalv;
	long	int	hnueva;
	float			diam;

	//	2006-02-22
	float			percent;
	long	int	update;
	
	totalv	=	0;
	
	Mesh2Estructura(objmesh, meshLeaves);
	
	diam = DiametroEsferaEnvolvente();

	EstableceCriterio(diam);

	float paramlod = lodfactor * activas * 0.01f;

	// El paramlod lo uso como condicin de parada de la simplificacin
	if (paramlod >= 6)
	{
		//	2006-02-22
		update	=	long((activas - (6 * paramlod)) / 80);
		percent	=	0.5;
		
		while ( activas > (6*paramlod))
		{
			//	2006-02-22
			if (mUPB	&&	(((long int)(activas - (6 * paramlod)) % update) == 0))
			{
				mUPB(percent);
			}
			
			hnueva	=	Colapsa(diam);
			
			EstableceCriterio2(diam, hnueva);
		}
	}
	else
	{
		//	2006-02-22
		update	=	(activas - 6) / 80;
		percent	=	0.5;
		
		while ( activas > 6) 
		{
			//	2006-02-22
			if (mUPB	&&	(((activas - 6) % update) == 0))
			{
				mUPB(percent);
			}
			
			hnueva	=	Colapsa(diam);
			
			EstableceCriterio2(diam, hnueva);
		}
	}

	EscribeMesh(meshLeaves);
	
	//	2006-02-22
	//	Fit progress bar to 100%.
	if (mUPB)
	{
		mUPB(40);
	}
}

void TreeSimplifier::Mesh2Estructura(	const Mesh	*mesh,
																			Index				meshLeaves)
{
	// Calcular el nmero de vrtices
	long int countv=0;
	long int pos=0;
	long int v1, v2, v3;
	long int num_triangulo=0; // Identifica los tringulos en las hojas

	//	2006-02-21.
	float			percent;
	long	int	update;
	
	countv+=(long int)mesh->mSubMesh[meshLeaves].mVertexBuffer->mVertexCount;
	Vertex  = new float[2*countv][3];
	
	//	2006-02-21.
	update	=	long(mesh->mSubMesh[meshLeaves].mVertexBuffer->mVertexCount / 20);
	percent	=	0.5;
	
	for (unsigned int j=0; j<mesh->mSubMesh[meshLeaves].mVertexBuffer->mVertexCount; j++)
	{
		//	2006-02-21.
		if (mUPB	&&	((j % update) == 0))
		{
			mUPB(percent);
		}
		
		Vertex[pos][0] = mesh->mSubMesh[meshLeaves].mVertexBuffer->mPosition[j].x; 
		Vertex[pos][1] = mesh->mSubMesh[meshLeaves].mVertexBuffer->mPosition[j].y; 
		Vertex[pos][2] = mesh->mSubMesh[meshLeaves].mVertexBuffer->mPosition[j].z;
		pos++;
	}

	// Calcular el nmero de hojas
	counth+=(long)mesh->mSubMesh[meshLeaves].mIndexCount;
	counth=counth/6; // Se supone que cada 6 ndices forman una hoja.

	if ( counth > 0)
	{
		Hojasa  = new Hoja[2*counth];
	}

	activas = counth;

	// Insertar las hojas en la estructura
	pos=0;

	update	=	long(mesh->mSubMesh[meshLeaves].mIndexCount	/ 20);
	percent	=	0.5;
	
	// Cada hoja son 6 vrtices
	for (unsigned int j=0; j<mesh->mSubMesh[meshLeaves].mIndexCount; j=j+6) 
	{
		//	2006-02-21.
		if (mUPB	&&	((j % update) == 0))
		{
			mUPB(percent);
		}
		
		// Primer tringulo
		v1=mesh->mSubMesh[meshLeaves].mIndex[j];
		v2=mesh->mSubMesh[meshLeaves].mIndex[j+1];
		v3=mesh->mSubMesh[meshLeaves].mIndex[j+2];
		Hojasa[pos].Vert_Hoja[0]=v1;
		Hojasa[pos].Vert_Hoja[1]=v2;
		Hojasa[pos].Vert_Hoja[2]=v3;
		Hojasa[pos].id_triangulo[0]=num_triangulo;
		Hojasa[pos].existe=true;
		num_triangulo++;

		// Segundo tringulo
		v3=mesh->mSubMesh[meshLeaves].mIndex[j+5];
		Hojasa[pos].Vert_Hoja[3]=v3;
		Hojasa[pos].id_triangulo[1]=num_triangulo;
		num_triangulo++;

		Centroh(Hojasa[pos]);
		GetNormal(Hojasa[pos]);
		pos++;
	}
}

float TreeSimplifier::max(float a,	float b)
{
	if (a>b) return (a);
	else return(b);
}

float TreeSimplifier::min(float a,	float b)
{
	if (a>b) return (b);
	else return(a);
}

float TreeSimplifier::distan(	float	x1,	float	y1,	float	z1,
															float x2,	float y2,	float z2)
{
	float dist = 0;

	dist = ((x2-x1)*(x2-x1)) + ((y2-y1)*(y2-y1)) + ((z2-z1)*(z2-z1));

	return ( dist);
}

//--------------------------------------------------------------------------------------------------------------------------------
//  CALCULA EL CENTRO DE UNA HOJA
//--------------------------------------------------------------------------------------------------------------------------------


void TreeSimplifier::Centroh(Hoja	&Hoja1)
{
	float	max_x;
	float	max_y;
	float	max_z;
	float	min_x;
	float	min_y;
	float	min_z;
	
	//x1
	max_x	=	max(max(Vertex[Hoja1.Vert_Hoja[0]][0],
									Vertex[Hoja1.Vert_Hoja[1]][0]),
				  		max(Vertex[Hoja1.Vert_Hoja[2]][0],
									Vertex[Hoja1.Vert_Hoja[3]][0]));
	
	min_x	=	min(min(Vertex[Hoja1.Vert_Hoja[0]][0],
									Vertex[Hoja1.Vert_Hoja[1]][0]),
				  		min(Vertex[Hoja1.Vert_Hoja[2]][0],
									Vertex[Hoja1.Vert_Hoja[3]][0]));

	Hoja1.Centro[0] = (max_x + min_x)/2;

	//y1
	max_y	=	max(max(Vertex[Hoja1.Vert_Hoja[0]][1],
									Vertex[Hoja1.Vert_Hoja[1]][1]),
				  		max(Vertex[Hoja1.Vert_Hoja[2]][1],
									Vertex[Hoja1.Vert_Hoja[3]][1]));
	
	min_y	=	min(min(Vertex[Hoja1.Vert_Hoja[0]][1],
									Vertex[Hoja1.Vert_Hoja[1]][1]),
				  		min(Vertex[Hoja1.Vert_Hoja[2]][1],
									Vertex[Hoja1.Vert_Hoja[3]][1]));

	Hoja1.Centro[1]	=	(max_y + min_y) / 2;
	
	//z1
	max_z	=	max(max(Vertex[Hoja1.Vert_Hoja[0]][2],
									Vertex[Hoja1.Vert_Hoja[1]][2]),
				  		max(Vertex[Hoja1.Vert_Hoja[2]][2],
									Vertex[Hoja1.Vert_Hoja[3]][2]));
	
	min_z	=	min(min(Vertex[Hoja1.Vert_Hoja[0]][2],
									Vertex[Hoja1.Vert_Hoja[1]][2]),
				  		min(Vertex[Hoja1.Vert_Hoja[2]][2],
									Vertex[Hoja1.Vert_Hoja[3]][2]));

	Hoja1.Centro[2]	=	(max_z + min_z) / 2;
}

//--------------------------------------------------------------------------------------------------------------------------------
//  CALCULA LA NORMAL DE UNA HOJA
//--------------------------------------------------------------------------------------------------------------------------------
void TreeSimplifier::GetNormal(Hoja &aHoja)
{
	float onex, oney, onez;
	float twox, twoy, twoz;
	float threex, threey, threez;

	onex=Vertex[aHoja.Vert_Hoja[0]][0]; oney= Vertex[aHoja.Vert_Hoja[0]][1]; onez = Vertex[aHoja.Vert_Hoja[0]][2];
	twox = Vertex[aHoja.Vert_Hoja[1]][0]; twoy = Vertex[aHoja.Vert_Hoja[1]][1]; twoz = Vertex[aHoja.Vert_Hoja[1]][2];
	threex = Vertex[aHoja.Vert_Hoja[2]][0]; threey = Vertex[aHoja.Vert_Hoja[2]][1]; threez = Vertex[aHoja.Vert_Hoja[2]][2];
	
	aHoja.Normal[0] = ((twoz-onez)*(threey-oney)) - ((twoy-oney)*(threez-onez));
	aHoja.Normal[1] = ((twox-onex)*(threez-onez)) - ((threex-onex)*(twoz-onez));
	aHoja.Normal[2] = ((threex-onex)*(twoy-oney)) - ((twox-onex)*(threey-oney));

	
}
//--------------------------------------------------------------------------------------------------------------------------------
//  CALCULA LA distancia de Hausdorff ( distancia entre nubes de puntos)
//--------------------------------------------------------------------------------------------------------------------------------
float TreeSimplifier::Hausdorff(Hoja &aHoja, Hoja& h2)
{
	float onex, oney, onez;
	float twox, twoy, twoz;
	float threex, threey, threez;
	float fourx, foury, fourz;
	float x1, y1, z1;
	float x2, y2, z2;
	float x3, y3, z3;
	float x4, y4, z4;
	float dist1, dist2, dist3, dist4, distmp, dista, distb, dist;

	onex=Vertex[aHoja.Vert_Hoja[0]][0]; oney= Vertex[aHoja.Vert_Hoja[0]][1]; onez = Vertex[aHoja.Vert_Hoja[0]][2];
	twox = Vertex[aHoja.Vert_Hoja[1]][0]; twoy = Vertex[aHoja.Vert_Hoja[1]][1]; twoz = Vertex[aHoja.Vert_Hoja[1]][2];
	threex = Vertex[aHoja.Vert_Hoja[2]][0]; threey = Vertex[aHoja.Vert_Hoja[2]][1]; threez = Vertex[aHoja.Vert_Hoja[2]][2];
	fourx = Vertex[aHoja.Vert_Hoja[3]][0]; foury = Vertex[aHoja.Vert_Hoja[3]][1]; fourz = Vertex[aHoja.Vert_Hoja[3]][2];
	

	x1 = Vertex[h2.Vert_Hoja[0]][0]; y1 = Vertex[h2.Vert_Hoja[0]][1]; z1 = Vertex[h2.Vert_Hoja[0]][2];
	x2 = Vertex[h2.Vert_Hoja[1]][0]; y2 = Vertex[h2.Vert_Hoja[1]][1]; z2 = Vertex[h2.Vert_Hoja[1]][2];
	x3 = Vertex[h2.Vert_Hoja[2]][0]; y3 = Vertex[h2.Vert_Hoja[2]][1]; z3 = Vertex[h2.Vert_Hoja[2]][2];
	x4 = Vertex[h2.Vert_Hoja[3]][0]; y4 = Vertex[h2.Vert_Hoja[3]][1]; z4 = Vertex[h2.Vert_Hoja[3]][2];

	// guardo las distancias mnimas de cada vrtice a los 4 contrarios.
	dist1 = distan ( onex, oney,onez,x1,y1,z1);
	// vertice 1 con los 4 de la otra hoja

	distmp = distan ( onex, oney,onez,x2,y2,z2);
	if ( distmp < dist1) dist1 = distmp;

	distmp = distan ( onex, oney,onez,x3,y3,z3);
	if ( distmp < dist1) dist1 = distmp;

	distmp = distan ( onex, oney,onez,x4,y4,z4);
	if ( distmp < dist1) dist1 = distmp;

	// vertice 2 con los 4 de la otra hoja

	dist2 = distan ( twox, twoy,twoz,x1,y1,z1);
	
	distmp = distan ( twox, twoy,twoz,x2,y2,z2);
	if ( distmp < dist2) dist2 = distmp;
	
	distmp = distan ( twox, twoy,twoz,x3,y3,z3);
	if ( distmp < dist2) dist2 = distmp;
	
	distmp = distan ( twox, twoy,twoz,x4,y4,z4);
	if ( distmp < dist2) dist2 = distmp;

	// vertice 3 con los 4 de la otra hoja

	dist3 = distan ( threex, threey,threez,x1,y1,z1);
	
	distmp = distan ( threex, threey,threez,x2,y2,z2);
	if ( distmp < dist3) dist3 = distmp;

	distmp = distan ( threex, threey,threez,x3,y3,z3);
	if ( distmp < dist3) dist3 = distmp;

	distmp = distan ( threex, threey,threez,x4,y4,z4);
	if ( distmp < dist3) dist3 = distmp;


	// vertice 4 con los 4 de la otra hoja

	dist4 = distan ( fourx, foury,fourz,x1,y1,z1);
	
	distmp = distan ( fourx, foury,fourz,x2,y2,z2);
	if ( distmp < dist4) dist4 = distmp;

	distmp = distan ( fourx, foury,fourz,x3,y3,z3);
	if ( distmp < dist4) dist4 = distmp;
	
	distmp = distan ( fourx, foury,fourz,x4,y4,z4);
	if ( distmp < dist4) dist4 = distmp;

	//de entre estos cojo el mximo

	dista =  max(dist1, max(dist2, max(dist3, dist4)));

	//LO MISMO PERO A LA INVERSA

	dist1 = distan ( x1,y1,z1, onex, oney, onez);
	// vertice 1 con los 4 de la otra hoja

	distmp = distan ( x1,y1,z1, twox, twoy, twoz);
	if ( distmp < dist1) dist1 = distmp;

	distmp = distan ( x1,y1,z1, threex, threey, threez);
	if ( distmp < dist1) dist1 = distmp;

	distmp = distan ( x1,y1,z1, fourx, foury, fourz);
	if ( distmp < dist1) dist1 = distmp;

	//2
	dist2 = distan ( x2,y2,z2, onex, oney, onez);
	// vertice 2 con los 4 de la otra hoja

	distmp = distan ( x2,y2,z2, twox, twoy, twoz);
	if ( distmp < dist2) dist2 = distmp;

	distmp = distan ( x2,y2,z2, threex, threey, threez);
	if ( distmp < dist2) dist2 = distmp;

	distmp = distan ( x2,y2,z2, fourx, foury, fourz);
	if ( distmp < dist2) dist2 = distmp;


		//3
	dist3 = distan ( x3,y3,z3, onex, oney, onez);
	// vertice 3 con los 4 de la otra hoja

	distmp = distan ( x3,y3,z3, twox, twoy, twoz);
	if ( distmp < dist3) dist3 = distmp;

	distmp = distan ( x3,y3,z3, threex, threey, threez);
	if ( distmp < dist3) dist3 = distmp;

	distmp = distan ( x3,y3,z3, fourx, foury, fourz);
	if ( distmp < dist3) dist3 = distmp;

		//4
	dist4 = distan ( x4,y4,z4, onex, oney, onez);
	// vertice 4 con los 4 de la otra hoja

	distmp = distan ( x4,y4,z4, twox, twoy, twoz);
	if ( distmp < dist4) dist4 = distmp;

	distmp = distan ( x4,y4,z4, threex, threey, threez);
	if ( distmp < dist4) dist4 = distmp;

	distmp = distan ( x4,y4,z4, fourx, foury, fourz);
	if ( distmp < dist4) dist4 = distmp;

	//
	distb =  max(dist1, max(dist2, max(dist3, dist4)));

	dist  = max ( dista, distb);
	return ( dist);
	
}

//--------------------------------------------------------------------------------------------------------------------------------
// CALCULA LA DISTANCIA ENTRE HOJAS
//--------------------------------------------------------------------------------------------------------------------------------

void TreeSimplifier::DistanciaEntreHojas(void)
{
	float dist, distmp ;
	int j;

	for ( int i=0;i<counth;i++)
	{
		if ( Hojasa[i].existe == true) // si la hoja aun existe
		{
			// inicializo para poder comparar
			if ( i == (counth-1)) j = 0; 
				else j = i+1;
			while (Hojasa[j].existe == false) j++;
			//dist = Hojasa[i].Distancia(Hojasa[j]);
			dist = Hausdorff( Hojasa[i], Hojasa[j]);

			Hojasa[i].hoja_cerca = j;
			// empiezo los calculos
			for ( j =0; j<(counth-1);j++)
			{
				if ( j == i) 
					break;
				if ( Hojasa[j].existe == true) // si la hoja aun existe
				{
					distmp = Hausdorff(Hojasa[i], Hojasa[j]);
					if ( distmp < dist ) 
					{
						dist = distmp;
						Hojasa[i].hoja_cerca = j;
					}
				}
				
			}
			Hojasa[i].dist = dist;
		}
	}

}

//--------------------------------------------------------------------------------------------------------------------------------
// DEVUELVE LA HOJA QUE TIENE UNA DISTANCIA MENOR
//--------------------------------------------------------------------------------------------------------------------------------
long int TreeSimplifier::MinimaDistancia ( void)
{
	float			mindist;
	long int	cual;
	int				i;

	mindist =	0;
	cual		=	-1;
	i				=	0;
	
	//inicializo
	while (Hojasa[i].existe != true)
	{
		i++;
	}
	
	mindist =	Hojasa[i].dist;
	cual		=	i;
	
	//busco la minima
	for (i = 0; i < counth; i++)
	{
		if (Hojasa[i].existe == true)
		{
			if ( mindist > Hojasa[i].dist)
			{
				mindist = Hojasa[i].dist;
				cual = i;
			}
		}

	}
	return (cual);
}

//--------------------------------------------------------------------------------------------------------------------------------
// CALCULA LA COPLANARIDAD ENTRE HOJAS
//--------------------------------------------------------------------------------------------------------------------------------
void TreeSimplifier::CoplanarEntreHojas(FILE	*fp_coplanar)
{
	float	cop;
	float	coptmp;
	int 	i;
	int 	j;

	for ( i=0;i<counth;i++)
	{
		if ( Hojasa[i].existe == true) // si la hoja aun existe
		{
			// inicializo para poder comparar
			if ( i == (counth-1))
			{
				j	=	0;
			}
			else
			{
				j =	i + 1;
			}
			
			while (Hojasa[j].existe == false)
			{
				j++;
			}
			
			cop									=	Hojasa[i].Coplanaridad(Hojasa[j]);
			Hojasa[i].hoja_cop	=	j;
			
			// empiezo los calculos
			for (j = 0; j < (counth-1); j++)
			{
				// si la hoja aun existe.
				if (( j != i) && (Hojasa[j].existe == true))
				{
					coptmp	=	Hojasa[i].Coplanaridad(Hojasa[j]);

					// COJO EL MAS COPLANAR , CERCANO A 1.
					if (coptmp > cop)
					{
						cop									=	coptmp;
						Hojasa[i].hoja_cop	=	j;
					}
				}
			}
			Hojasa[i].coplanar	=	1 - cop; //  8/6/01 LO INVIERTO
		}
	}
}

//--------------------------------------------------------------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------------------------------------------------------------
void TreeSimplifier::DosMayores(float *mayores, int	*indices)
{
	float	m1;
	int		i;

	if	(mayores[0] < mayores[1])
	{
		m1					=	mayores[0];
		mayores[0]	=	mayores[1];
		mayores[1]	=	m1;

		i						=	indices[0];
		indices[0]	=	indices[1];
		indices[1]	=	i;
	}

	if (mayores [2] < mayores [3])
	{
		m1					=	mayores[2];
		mayores[2]	=	mayores[3];
		mayores [3]	=	m1;

		i						=	indices[2];
		indices[2]	=	indices[3];
		indices[3]	=	i;
	}

	if  (mayores[0] < mayores[2])
	{
		m1					=	mayores[0];
		mayores[0]	=	mayores[2];
		mayores[2]	=	m1;

		i						=	indices[0];
		indices[0]	=	indices[2];
		indices[2]	=	i;
	}

	if (mayores [2] < mayores [3])
	{
		m1					=	mayores[2];
		mayores[2]	=	mayores[3];
		mayores [3]	=	m1;

		i						=	indices[2];
		indices[2]	=	indices[3];
		indices[3]	=	i;
	}

	if (mayores [1] < mayores [2])
	{
		m1					=	mayores[1];
		mayores[1]	=	mayores[2];
		mayores [2]	=	m1;

		i						= indices[1];
		indices[1]	= indices[2];
		indices[2]	= i;
	}		
}

//--------------------------------------------------------------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------------------------------------------------------------
//float TreeSimplifier::DiametroEsferaEnvolvente (FILE* fp_simplifica)
float TreeSimplifier::DiametroEsferaEnvolvente()
{
	float xmax, xmin, ymax, ymin, zmax, zmin;
	float diametro, cx, cy, cz;
	int j;

	//	2006-02-21.
	float			percent;
	long	int	update;

	//inicializo al primero de los vrtices de las hojas

	xmax = Vertex[Hojasa[0].Vert_Hoja[0]][0];
	xmin = xmax;

	ymax = Vertex[Hojasa[0].Vert_Hoja[0]][1];
	ymin = ymax;

	zmax = Vertex[Hojasa[0].Vert_Hoja[0]][2];
	zmin = zmax;

	// ahora busco los maximos y los minimos correspondientes


	//	2006-02-21
	update	=	counth / 20;
	percent	=	0.5;
	for (int i = 1; i < counth; i++)
	{
		//	2006-02-21
		if (mUPB	&&	((i % update) == 0))
		{
			mUPB(percent);
		}
		
		for ( j=0;j<4;j++)
		{
			
		if ( xmax < Vertex[Hojasa[i].Vert_Hoja[j]][0]) xmax = Vertex[Hojasa[i].Vert_Hoja[j]][0];
		if ( xmin > Vertex[Hojasa[i].Vert_Hoja[j]][0]) xmin = Vertex[Hojasa[i].Vert_Hoja[j]][0];

		if ( ymax < Vertex[Hojasa[i].Vert_Hoja[j]][1]) ymax = Vertex[Hojasa[i].Vert_Hoja[j]][1];
		if ( ymin > Vertex[Hojasa[i].Vert_Hoja[j]][1]) ymin = Vertex[Hojasa[i].Vert_Hoja[j]][1];
		
		if ( zmax < Vertex[Hojasa[i].Vert_Hoja[j]][2]) zmax = Vertex[Hojasa[i].Vert_Hoja[j]][2];
		if ( zmin > Vertex[Hojasa[i].Vert_Hoja[j]][2]) zmin = Vertex[Hojasa[i].Vert_Hoja[j]][2];

		}
	}

	cx = (xmax + xmin)/2;
	cy = (ymax + ymin)/2;
	cz = (zmax + zmin)/2;


	diametro = ((xmax-xmin)*(xmax-xmin)) + ((ymax-ymin)*(ymax-ymin)) + ((zmax-zmin)*(zmax-zmin));

return (diametro);
}

//--------------------------------------------------------------------------------------------------------------------------------
//  normaliza las distancia
//--------------------------------------------------------------------------------------------------------------------------------
void TreeSimplifier::NormalizaDistancia (float diametro)
{
	float dtmp;
	for (int i=0; i<counth;i++)
	{
		dtmp = Hojasa[i].dist;
		Hojasa[i].dist = dtmp / diametro;
		
	}
}

//--------------------------------------------------------------------------------------------------------------------------------
//  establece el criterio como (K1 * dist + K2 * cop)/ K1+K2
//--------------------------------------------------------------------------------------------------------------------------------
void TreeSimplifier::EstableceCriterio(float diametro)
{
	//	Para empezar establezco K1 y K2 con 0,5
	float coptmp2, coptmp, distmp2, distmp, criteriotmp;
	int i, j, nhojasi, nhojasj;

	//	2006-02-21
	float			percent;
	long	int	update;
	
	update	=	counth / 30;
	percent	=	1;
	
	for ( i=0; i<counth;i++)
	{
		//	2005-02-21
		if (mUPB	&&	((i % update) == 0))
		{
			mUPB(percent);
		}
		
		if (Hojasa[i].existe == true)
		{
			//incializo criterio a un numero elevado
			Hojasa[i].criterio = 1000;
			nhojasi = int(Hojasa[i].Cuantas_hojas);
			//coplanaridad
			for ( j =0; j<counth;j++)
			{
				if (( j != i) && ( Hojasa[j].existe == true)) // si la hoja aun existe
				{
					//17/09/01 ANTES DE CALCULAR NADA, COMPRUEBO QUE ESTAS DOS HOJAS 
					// SE PODRIAN COLAPSAR, ED, QUE LA DIFERENCIA DE HOJAS QUE COLAPSAN
					// ES COMO MXIMO 1

					nhojasj = int(Hojasa[j].Cuantas_hojas);

					if ( abs((nhojasi - nhojasj)) < 2)
					{
						//coplanaridad y lo invierto
						coptmp2 = Hojasa[i].Coplanaridad(Hojasa[j]); 
						coptmp = 1 - coptmp2;
						//distancia y la normalizo
						distmp2 = Hausdorff( Hojasa[i], Hojasa[j]); 
						distmp = distmp2 / diametro;
						// calculo el criterio para esa hoja
						criteriotmp = (( K1 * distmp * distmp ) + (K2 * coptmp * distmp))/ (K1 + K2);
						//selecciono el criterio menor
						if (Hojasa[i].criterio > criteriotmp) 
						{
							Hojasa[i].criterio = criteriotmp;
							Hojasa[i].hoja_crit = j;
						}
					}
				}
				
			}
		}
	}
}

//--------------------------------------------------------------------------------------------------------------------------------
//  establece el criterio despues de colapsar
//--------------------------------------------------------------------------------------------------------------------------------
void TreeSimplifier::EstableceCriterio2 ( float			diametro,
																					long int	hojanueva)
{
	//Para empezar establezco K1 y K2 con 0,5
	float  coptmp2, coptmp, distmp2, distmp, criteriotmp;
	int i, j, nhojasi, nhojasj;
	//ESTAN EN tres PROCEDIMIENTOS: COLAPSA, ESTABLECECRITERIO Y ESTABLECECRITERIO2
	
	for (i = 0; i < counth; i++)
	{
		if ((Hojasa[i].existe == true) && (i != hojanueva))
		{	
			nhojasi = int(Hojasa[i].Cuantas_hojas);
			// SE HA DESACTIVADO LA HOJA_CRIT QUE GUARDABA LA HOJA?
			if ( Hojasa[Hojasa[i].hoja_crit].existe == false)
			{
				Hojasa[i].criterio = 1000;

				//coplanaridad
				for ( j =0; j<counth;j++)
				{
					if (( j != i) && ( Hojasa[j].existe == true)) // si la hoja aun existe
					{
						//17/09/01 ANTES DE CALCULAR NADA, COMPRUEBO QUE ESTAS DOS HOJAS 
						// SE PODRIAN COLAPSAR, ED, QUE LA DIFERENCIA DE HOJAS QUE COLAPSAN
						// ES COMO MXIMO 1

						nhojasj = int(Hojasa[j].Cuantas_hojas);

						if ( abs((nhojasi - nhojasj)) < 2)
						{

							//coplanaridad y lo invierto
							coptmp2 = Hojasa[i].Coplanaridad(Hojasa[j]); 
							coptmp = 1 - coptmp2;	
							//distancia y la normalizo	
						//	distmp2 = Hojasa[i].Distancia(Hojasa[j]); 
							distmp2 = Hausdorff( Hojasa[i],  Hojasa[j]); 
							distmp = distmp2 / diametro;
							// calculo el criterio para esa hoja
							criteriotmp = (( K1 * distmp * distmp ) + (K2 * coptmp * distmp))/ (K1 + K2);
							//selecciono el criterio menor
							if (Hojasa[i].criterio > criteriotmp) 
							{
								Hojasa[i].criterio = criteriotmp;
								Hojasa[i].hoja_crit = j;
							}
						}
					}
				}
			}
			else
			{ // CALCULARE SI EL CRITERIO CON ESTA HOJA ES MENOR QUE EL ANTERIOR
				nhojasj = int(Hojasa[hojanueva].Cuantas_hojas);

				if ( abs((nhojasi - nhojasj)) < 2)
				{
					//coplanaridad y lo invierto
					coptmp2 = Hojasa[i].Coplanaridad(Hojasa[hojanueva]); 
					coptmp = 1 - coptmp2;
					//distancia y la normalizo
					//distmp2 = Hojasa[i].Distancia(Hojasa[hojanueva]); 
					distmp2 = Hausdorff( Hojasa[i], Hojasa[hojanueva]); 
					distmp = distmp2 / diametro;
					// calculo el criterio para esa hoja
					criteriotmp = (( K1 * distmp * distmp ) + (K2 * coptmp * distmp))/ (K1 + K2);
					//selecciono el criterio menor
					if (Hojasa[i].criterio > criteriotmp) 
					{	
						Hojasa[i].criterio = criteriotmp;
						Hojasa[i].hoja_crit = hojanueva;
					}
				}
			}
		}
	}
}
//--------------------------------------------------------------------------------------------------------------------------------
// DEVUELVE LA HOJA QUE TIENE EL NUMERO EN EL CAMPO CRITERIO MENOR
//--------------------------------------------------------------------------------------------------------------------------------

long int TreeSimplifier::MinimoCriterio (void)
{
	float mincrit =0;
	long int cual=-1;
	int i=0;
	
	//inicializo
	while (Hojasa[i].existe != true) i++;
	mincrit = Hojasa[i].criterio;
	cual =i;
	//busco la minima
	for ( i=0;i<counth;i++)

	{
		if (Hojasa[i].existe == true)
		{
			if ( mincrit> Hojasa[i].criterio)
			{
				mincrit = Hojasa[i].criterio;
				cual = i;
			}
		}
		
	}
	return (cual);
}


//--------------------------------------------------------------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------------------------------------------------------------
void TreeSimplifier::ElijeVertices(Hoja& Hoja1, Hoja& Hoja2, long int count)
{
	// criterio es el de los dos vertices ms alejados del centro de la otra hoja
	
	float a,b,c;
	float dist[4];
	int indices[4];
	
	// primero de la primera hoja cojo los dos primeros vertices


	a = Vertex[Hoja1.Vert_Hoja[0]][0];
	b = Vertex[Hoja1.Vert_Hoja[0]][1];
	c = Vertex[Hoja1.Vert_Hoja[0]][2];

	dist[0] = ((Hoja2.Centro[0]-a)*(Hoja2.Centro[0]-a)) + ((Hoja2.Centro[1]-b)*(Hoja2.Centro[1]-b)) +
		   ((Hoja2.Centro[2]-c)*(Hoja2.Centro[2]-c));

	
	a = Vertex[Hoja1.Vert_Hoja[1]][0];
	b = Vertex[Hoja1.Vert_Hoja[1]][1];
	c = Vertex[Hoja1.Vert_Hoja[1]][2];

	dist[1] = ((Hoja2.Centro[0]-a)*(Hoja2.Centro[0]-a)) + ((Hoja2.Centro[1]-b)*(Hoja2.Centro[1]-b)) +
		   ((Hoja2.Centro[2]-c)*(Hoja2.Centro[2]-c));

	a = Vertex[Hoja1.Vert_Hoja[2]][0];
	b = Vertex[Hoja1.Vert_Hoja[2]][1];
	c = Vertex[Hoja1.Vert_Hoja[2]][2];

	dist[2] = ((Hoja2.Centro[0]-a)*(Hoja2.Centro[0]-a)) + ((Hoja2.Centro[1]-b)*(Hoja2.Centro[1]-b)) +
		   ((Hoja2.Centro[2]-c)*(Hoja2.Centro[2]-c));


	a = Vertex[Hoja1.Vert_Hoja[3]][0];
	b = Vertex[Hoja1.Vert_Hoja[3]][1];
	c = Vertex[Hoja1.Vert_Hoja[3]][2];

	dist[3] = ((Hoja2.Centro[0]-a)*(Hoja2.Centro[0]-a)) + ((Hoja2.Centro[1]-b)*(Hoja2.Centro[1]-b)) +
		   ((Hoja2.Centro[2]-c)*(Hoja2.Centro[2]-c));

	for ( int i=0;i<4;i++) indices[i]=i;

	DosMayores(dist, indices);
	
	Hojasa[counth].Vert_Hoja[0] = Hoja1.Vert_Hoja[indices[0]]; 
	Hojasa[counth].Vert_Hoja[1] = Hoja1.Vert_Hoja[indices[1]]; 
	
	// segunda hoja los dos ultimos vertices

	
	a = Vertex[Hoja2.Vert_Hoja[0]][0];
	b = Vertex[Hoja2.Vert_Hoja[0]][1];
	c = Vertex[Hoja2.Vert_Hoja[0]][2];

	dist[0] = ((Hoja1.Centro[0]-a)*(Hoja1.Centro[0]-a)) + ((Hoja1.Centro[1]-b)*(Hoja1.Centro[1]-b)) +
		   ((Hoja1.Centro[2]-c)*(Hoja1.Centro[2]-c));

	
	a = Vertex[Hoja2.Vert_Hoja[1]][0];
	b = Vertex[Hoja2.Vert_Hoja[1]][1];
	c = Vertex[Hoja2.Vert_Hoja[1]][2];

	dist[1] = ((Hoja2.Centro[0]-a)*(Hoja2.Centro[0]-a)) + ((Hoja1.Centro[1]-b)*(Hoja1.Centro[1]-b)) +
		   ((Hoja2.Centro[2]-c)*(Hoja2.Centro[2]-c));

	a = Vertex[Hoja2.Vert_Hoja[2]][0];
	b = Vertex[Hoja2.Vert_Hoja[2]][1];
	c = Vertex[Hoja2.Vert_Hoja[2]][2];

	dist[2] = ((Hoja1.Centro[0]-a)*(Hoja1.Centro[0]-a)) + ((Hoja1.Centro[1]-b)*(Hoja1.Centro[1]-b)) +
		   ((Hoja1.Centro[2]-c)*(Hoja1.Centro[2]-c));


	a = Vertex[Hoja2.Vert_Hoja[3]][0];
	b = Vertex[Hoja2.Vert_Hoja[3]][1];
	c = Vertex[Hoja2.Vert_Hoja[3]][2];

	dist[3] = ((Hoja1.Centro[0]-a)*(Hoja1.Centro[0]-a)) + ((Hoja1.Centro[1]-b)*(Hoja1.Centro[1]-b)) +
		   ((Hoja1.Centro[2]-c)*(Hoja1.Centro[2]-c));

	for ( int i=0;i<4;i++) indices[i]=i;

	DosMayores(dist, indices);
	
	Hojasa[counth].Vert_Hoja[2] = Hoja2.Vert_Hoja[indices[0]]; 
	Hojasa[counth].Vert_Hoja[3] = Hoja2.Vert_Hoja[indices[1]]; 



	
}

//--------------------------------------------------------------------------------------------------------------------------------
// AADE HOJAS 
//--------------------------------------------------------------------------------------------------------------------------------
//long int TreeSimplifier::Colapsa (Geometry::TreeSimplificationSequence *tss, float diametro)
long int TreeSimplifier::Colapsa(float diametro)
{
	long int i=0, cual=-1;
	long int otra = -1;
	float coptmp, coptmp2, distmp, distmp2, criteriotmp;
//	char linea[250];

//	cual=MinimaDistancia();
	cual=MinimoCriterio();
//	otra = Hojasa[cual].hoja_cerca; //para la distancia
//	otra = Hojasa[cual].hoja_cop;
	otra = Hojasa[cual].hoja_crit;

	//desactivo las hojas cercanas
	Hojasa[cual].existe = false;
	Hojasa[otra].existe = false;
	//creo la hoja nueva

	Hojasa[counth].hoja_cerca = -1;
	Hojasa[counth].dist = 0;
	Hojasa[counth].hoja_cop = -1;
	Hojasa[counth].coplanar = 0;

	Hojasa[counth].Cuantas_hojas = Hojasa[cual].Cuantas_hojas + Hojasa[otra].Cuantas_hojas;
	ElijeVertices(Hojasa[cual], Hojasa[otra], counth);
	Centroh (Hojasa[counth]);
	GetNormal ( Hojasa[counth]);

	//18/09/01 
	if (Hojasa[counth].Cuantas_hojas > 60 )
	{
			Hojasa[counth].existe = false;
			activas--;
	}
	else
	{
		//establezco el criterio para la hoja nueva
     	Hojasa[counth].existe = true;
		Hojasa[counth].criterio = 1000;
		Hojasa[counth].id_triangulo[0] = counth*2;
		Hojasa[counth].id_triangulo[1] = counth*2+1;

 		//coplanaridad
		for (int j = 0; j < (counth+1); j++)
		{
			if (( j != counth) && ( Hojasa[j].existe == true)) // si la hoja aun existe
			{
			//coplanaridad y lo invierto
			coptmp2 = Hojasa[counth].Coplanaridad(Hojasa[j]); 
			coptmp = 1 - coptmp2;
			//distancia y la normalizo
			//distmp2 = Hojasa[counth].Distancia(Hojasa[j]); 
			distmp2 = Hausdorff(Hojasa[counth], Hojasa[j]); 
			distmp = distmp2 / diametro;
			// calculo el criterio para esa hoja
			criteriotmp = (( K1 * distmp * distmp ) + (K2 * coptmp * distmp))/ (K1 + K2);
			//selecciono el criterio menor
			if (Hojasa[counth].criterio > criteriotmp) 
			{
				Hojasa[counth].criterio = criteriotmp;
				Hojasa[counth].hoja_crit = j;}
			}
		}
	}	

	// Crear el paso de simplificacin
	Geometry::TreeSimplificationSequence::Step pasosimp;
	pasosimp.mV0=Hojasa[cual].id_triangulo[0];
	pasosimp.mV1=Hojasa[cual].id_triangulo[1];
	pasosimp.mT0=Hojasa[otra].id_triangulo[0];
	pasosimp.mT1=Hojasa[otra].id_triangulo[1];

	// Nuevos vrtices
	pasosimp.mNewQuad[0]=Hojasa[counth].Vert_Hoja[0];
	pasosimp.mNewQuad[1]=Hojasa[counth].Vert_Hoja[1];
	pasosimp.mNewQuad[2]=Hojasa[counth].Vert_Hoja[2];
	pasosimp.mNewQuad[3]=Hojasa[counth].Vert_Hoja[3];

	// Insertar el paso de simplificacin
	mtreesimpsequence->mSteps.push_back(pasosimp);

	//incremento el numero de hojas
	counth++;
	activas--;
	return (counth-1);//porque lo he incrementado en la linea de antes
}

//--------------------------------------------------------------------------------------------------------------------------------
// ESCRIBE EL mIndex EN EL MESH DE SALIDA
//--------------------------------------------------------------------------------------------------------------------------------
void TreeSimplifier::EscribeMesh(int idMeshLeaves) 
{
	// Calcular el nmero de indices despus de simplificar
	long int tamIndex	=	activas * 6; // Cada hoja tiene 6 ndices

	//	2006-02-21.
	float			percent;
	long	int	update;
	
	delete [] mesh->mSubMesh[idMeshLeaves].mIndex;
	mesh->mSubMesh[idMeshLeaves].mIndexCount=tamIndex;
	mesh->mSubMesh[idMeshLeaves].mIndex=new Geometry::Index[tamIndex];

	// Recorrer las hojas, comprobar las que estn activas y copiar los ndices al mesh.
	Geometry::Index indice=0;

	//	2006-02-21.
	update	=	counth	/ 10;
	percent	=	1;
	
	for (long j=0; j<counth;j++)
	{
		//	2006-02-21
		if (mUPB	&&	((j % update) == 0))
		{
			mUPB(percent);
		}
		
		if ( Hojasa[j].existe == true) // si la hoja aun existe
		{
			mesh->mSubMesh[idMeshLeaves].mIndex[indice]=Hojasa[j].Vert_Hoja[0];
			mesh->mSubMesh[idMeshLeaves].mIndex[indice+1]=Hojasa[j].Vert_Hoja[1];
			mesh->mSubMesh[idMeshLeaves].mIndex[indice+2]=Hojasa[j].Vert_Hoja[2];
			mesh->mSubMesh[idMeshLeaves].mIndex[indice+3]=Hojasa[j].Vert_Hoja[2];
			mesh->mSubMesh[idMeshLeaves].mIndex[indice+4]=Hojasa[j].Vert_Hoja[1];
			mesh->mSubMesh[idMeshLeaves].mIndex[indice+5]=Hojasa[j].Vert_Hoja[3];
			indice=indice+6;
		}
	}
}
